// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include <deque>
#include <mutex>
#include "xmacpp/FilterSession.h"
#include "SessionBase.h"
#include "FilterThread.h"
#include "BufferPool.h"

namespace xma {
//////////////////////////////////////////////////////////////////////////////
#define XMA_DMA_DEFAULT_DELAY 18

class InternalDmaFilterSession final : virtual public FilterSession, virtual public FilterThread<> {
public:
  InternalDmaFilterSession(IntSessionFactoryPtr sf, XmaFilterProperties* props);
  virtual ~InternalDmaFilterSession();

  virtual int32_t Enqueue(queue_t& data) override;
  virtual int32_t Dequeue(queue_t& data) override;

protected:
  virtual int32_t Process(queue_t& in, queue_t& out, EmptyDynamicParams* dynamic_params) override;

private:
  FramePtr CopyToHost(FramePtr src);
  FramePtr CopyToDevice(FramePtr src);

protected:
  int                m_dir = -1;
  FramePropertiesPtr m_temp_props;
  FramePtr           m_temp;
  BufferPoolPtr      m_pool;
};

using IntDmaFilterSessionPtr = std::shared_ptr<InternalDmaFilterSession>;

}; // Namespace xma
